package sample.application;

import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class MainServlet extends HttpServlet {

  private static final long serialVersionUID = 1L;

  private static Logger log;
  static {
    log4jConfigure();
  }

  public MainServlet() {
  }

  private static void log4jConfigure() {
    log = Logger.getLogger("sample/application/MainServlet");
  }

  protected void process(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException {
    dispatch("/request.jsp", req, res);
    log.error("Starting (not actually an error)");
  }

  protected void dispatch(String uri, HttpServletRequest req,
      HttpServletResponse res) {
    ServletContext sc = null;
    RequestDispatcher rd = null;
    sc = getServletContext();
    rd = sc.getRequestDispatcher(uri);
    try {
      rd.forward(req, res);
    } catch (ServletException e1) {
      e1.printStackTrace();
    } catch (IOException e2) {
      e2.printStackTrace();
    }

    Enumeration e = LogManager.getCurrentLoggers();
    while (e.hasMoreElements()) {
      Logger l = (Logger) e.nextElement();
      String name = l.getName();
      if (name != null) {
        System.out.println("logger name=" + name);
        if (isClassName(name)) {
          Class clazz = loadClassByName(name);
          if (clazz != null && isParentClassLoader(clazz)) {
            System.out.println(name + " is in a parent class loader of type "+clazz.getClassLoader().getClass().getName());
          }
        }
      }
    }
  }

  boolean isParentClassLoader(Class otherClass) {
    ClassLoader otherCL = otherClass.getClassLoader();
    ClassLoader thisCL = this.getClass().getClassLoader();
    ClassLoader parentCL = thisCL.getParent();
    while (parentCL != null) {
      if (parentCL == otherCL) {
        return true;
      }
      parentCL = parentCL.getParent();
    }
    return false;
  }

  boolean isClassName(String n) {
    if (n.contains("[") || n.contains("]") || n.contains("/")) {
      return false;
    }
    if (n.contains(".")) {
      return true;
    }
    return false;
  }

  Class loadClassByName(String className) {
    try {
      return Class.forName(className);
    } catch (ClassNotFoundException e1) {
      return null;
    } catch (Exception e) {
      e.printStackTrace(); // this is unexpected
      return null;
    }
  }

  @Override
  protected void doGet(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException {
    process(req, res);
  }

  @Override
  protected void doPost(HttpServletRequest req, HttpServletResponse res)
      throws ServletException, IOException {
    process(req, res);
  }
}
